@@ -15,10 +15,15 @@
  *                                                                         *
  ***************************************************************************/
 
+
+
+#include "default.h"
 #include "clamlibscanner.h"
 #include <sys/types.h>
 #include <sys/wait.h>
 
+#ifdef USECLAM
+
 //Init Clamav scanner engine
 bool ClamLibScanner::InitDatabase()
 {
@@ -110,7 +115,7 @@ int ClamLibScanner::Scanning( )
     if((ret = cl_scandesc(fd, &virname, &size, root, &limits, SCANOPTS)) == CL_VIRUS)
     {
 
-        LogFile::ErrorMessage ("Virus %s in file %s detect!\n", virname, FileName );
+        LogFile::ErrorMessage ("Virus %s in file %s detected!\n", virname, FileName );
 
         ScannerAnswer=virname;
 //PSEstart
@@ -177,6 +182,13 @@ int ClamLibScanner::ScanningComplete()
 
 }
 
+//PSEstart
+bool ClamLibScanner::FreeDatabase()
+{
+	cl_free(root);
+	return true;
+}
+//PSEend
 
 //Constructor
 ClamLibScanner::ClamLibScanner()
@@ -191,3 +203,5 @@ ClamLibScanner::ClamLibScanner()
 ClamLibScanner::~ClamLibScanner()
 {
 }
+
+#endif
