@@ -15,6 +15,8 @@
  *                                                                         *
  ***************************************************************************/
 
+#ifdef USECLAM
+
 #ifndef CLAMLIBSCANNER_H
 #define CLAMLIBSCANNER_H
 
@@ -39,6 +41,10 @@ bool ReloadDatabase();
 
 bool InitSelfEngine();
 
+//PSEstart
+bool FreeDatabase();
+//PSEend
+
 int ScanningComplete();
 
 int Scanning( );
@@ -48,3 +54,4 @@ int Scanning( );
 };
 
 #endif
+#endif
