@@ -16,59 +16,43 @@
  ***************************************************************************/
 
 #include "proxyhandler.h"
-#include <sys/ipc.h>
-#include <sys/msg.h>
+#include "whitelist.h"
 
+extern URLList Whitelist;
+extern URLList Blacklist;
 
 bool ProxyHandler::Proxy ( SocketHandler *ProxyServerT, GenericScanner *VirusScannerT )
 {
     int CommunicationAnswer;
-    //PSEstart
-    int msqid;
-    struct msqid_ds *buf;
-    //PSEend
 
     if ( (CommunicationAnswer = Communication ( ProxyServerT, VirusScannerT )) != 0)
     {
 
         //ScannningComplete was not called
-        if ( CommunicationAnswer < 200 ){
+        if (( CommunicationAnswer > -200 ) && ( CommunicationAnswer < 0 )) {
         VirusScannerT->ScanningComplete();
         }
 
    	ProxyMessage( CommunicationAnswer , VirusScannerT);
-        //PSE ProxyMessage( CommunicationAnswer );
 
         ToBrowser.Close();
         ToServer.Close();
 
-#ifdef QUEUE
-	//PSE: We have done our job => delete message-queue
-	msqid = VirusScannerT->msgqid;
-	if(msgctl(msqid,IPC_RMID,buf) < 0) {
-		LogFile::ErrorMessage("Cannot delete message queue! Error: %s\n",strerror(errno));
-   	}
-#endif
-
         return false;
     }
 
 
-#ifdef LOG_OKS
-    LogFile::AccessMessage("%s %d OK\n", ToBrowser.GetCompleteRequest(), ToBrowser.GetPort());
-#endif
+// #ifdef LOG_OKS
+    if(Params::GetConfigBool("LOG_OKS"))
+    LogFile::AccessMessage("%s %d %d %s OK\n", ToBrowser.GetCompleteRequest(), ToServer.GetResponse(), ToBrowser.GetPort(), ToBrowser.GetRequestType().c_str() );
+// #endif
 
+//PSE: withdraw Clean-message
+    string ans = VirusScannerT->ReadScannerAnswer();
 
     ToBrowser.Close();
     ToServer.Close();
 
-#ifdef QUEUE
-    //PSE: We have done our job => delete message-queue
-    msqid = VirusScannerT->msgqid;
-    if(msgctl(msqid,IPC_RMID,buf) < 0) {
-	LogFile::ErrorMessage("Cannot delete message queue! Error: %s\n",strerror(errno));
-    }
-#endif
     return true;
 }
 
@@ -78,6 +62,7 @@ bool ProxyHandler::Proxy ( SocketHandler *ProxyServerT, GenericScanner *VirusSca
 int ProxyHandler::Communication( SocketHandler *ProxyServerT, GenericScanner *VirusScannerT)
 {
 
+    bool ScannerOff;
     int TempScannerAnswer;
 
     bool unlock = false;
@@ -104,6 +89,10 @@ int ProxyHandler::Communication( SocketHandler *ProxyServerT, GenericScanner *Vi
     string BodyTemp;
     int TransferDataLength=0;
 
+    unsigned int maxscansize = Params::GetConfigInt("MAXSCANSIZE");
+    int TricklingTime = Params::GetConfigInt("TRICKLING");
+    unsigned int KeepBackBuffer = Params::GetConfigInt("KEEPBACKBUFFER");
+
     if( ProxyServerT->AcceptClient( &ToBrowser ) == false)
     {
         BrowserDropped = true;
@@ -124,19 +113,31 @@ int ProxyHandler::Communication( SocketHandler *ProxyServerT, GenericScanner *Vi
         return -30;
     }
 
-    #if defined (PARENTPROXY) && defined (PARENTPORT)
-    if( ToServer.SetDomainAndPort( PARENTPROXY, PARENTPORT ) == false )
+    ScannerOff = Whitelist.URLFound ( ToBrowser.GetHost(), ToBrowser.GetRequest() );
+
+    if ( Blacklist.URLFound ( ToBrowser.GetHost(), ToBrowser.GetRequest() ) == true ) {
+      LogFile::ErrorMessage("URL is blacklisted: %s\n", ToBrowser.GetCompleteRequest() );
+      return -45;
+    }
+ 
+    // #if defined (PARENTPROXY) && defined (PARENTPORT)
+    string parentproxy=Params::GetConfigString("PARENTPROXY");
+    int parentport=Params::GetConfigInt("PARENTPORT");
+    if( parentproxy != "" && parentport != 0 ) {
+    if( ToServer.SetDomainAndPort( parentproxy.c_str(), parentport ) == false )
     {
-        LogFile::ErrorMessage("Could not resolve parent proxy: %s\n", PARENTPROXY);
+        LogFile::ErrorMessage("Could not resolve parent proxy: %s\n", parentproxy.c_str() );
         return -50;
     }
-    #else
+    // #else
+    } else {
     if( ToServer.SetDomainAndPort( ToBrowser.GetHost(), ToBrowser.GetPort() ) == false )
     {
         LogFile::ErrorMessage("Could not resolve hostname: %s\n", ToBrowser.GetHost() );
         return -50;
     }
-    #endif
+    // #endif
+    }
 
     if( ToServer.ConnectToServer ( ) == false)
     {
@@ -233,19 +234,27 @@ int ProxyHandler::Communication( SocketHandler *ProxyServerT, GenericScanner *Vi
 
     ContentLengthReference = ToServer.GetContentLength( );
 
-    if ( ContentLengthReference > 0 )
+    //Also check if file is too large for scanning
+    if ( ( ContentLengthReference > 0 ) && ( ( ContentLengthReference < maxscansize ) || (maxscansize == 0) ) )
     {
         unlock = true;
         //Set tempfile to right size
         if ( VirusScannerT->SetFileSize( ContentLengthReference ) == false )
         {
-            LogFile::ErrorMessage("Could set file size: %s Port %d\n", ToBrowser.GetHost(), ToBrowser.GetPort());
+            LogFile::ErrorMessage("Could set file size - Check disk space: %s Port %d\n", ToBrowser.GetHost(), ToBrowser.GetPort());
             return -90;
         }
     }
 
     Header = ToServer.PrepareHeaderForBrowser();
 
+
+ //Not Modified no Body expected or HEAD with no body
+ if( (ToServer.GetResponse() != 304 ) && (ToServer.GetResponse() != -302 ) && (ToBrowser.GetRequestType() != "HEAD") ) {
+
+    //Start Time for Trickling
+    time_t LastTrickling = time(NULL);
+
     //Server Body Transfer
     while ( (BodyLength = ToServer.ReadBodyPart(&BodyTemp)) != 0)
     {
@@ -268,25 +277,70 @@ int ProxyHandler::Communication( SocketHandler *ProxyServerT, GenericScanner *Vi
 
         //Body size check
         ContentLength += BodyLength;
-        if ( (unlock == true) && ( ContentLength > ContentLengthReference ) )
+        if ( (unlock == true) && (ScannerOff != true) && ( ContentLength > ContentLengthReference ) )
         {
-            LogFile::ErrorMessage("ContentLength and Body size does not fit: %s Port %d\n", ToBrowser.GetHost(), ToBrowser.GetPort());
+            LogFile::ErrorMessage("ContentLength and Body size does not fit - body too long: %s Port %d\n", ToBrowser.GetHost(), ToBrowser.GetPort());
             return -120;
         }
 
+
         //Add string to queue
         BodyQueue.push_back( BodyTemp );
 
-        //Expand file to scan
-        if ( VirusScannerT->ExpandFile( (char *)BodyTemp.c_str(), BodyTemp.length() , unlock ) == false )
+        //File to large for scanning
+        if ( (maxscansize != 0) && ( ContentLength > maxscansize ) )
         {
-            LogFile::ErrorMessage("Could not expand tempfile: %s Port %d\n", ToBrowser.GetHost(), ToBrowser.GetPort());
+         ScannerOff = true;
+        }
+
+        if (ScannerOff != true) 
+        {
+          //Expand file to scan
+          if ( VirusScannerT->ExpandFile( (char *)BodyTemp.c_str(), BodyTemp.length() , unlock ) == false )
+          {
+            LogFile::ErrorMessage("Could not expand tempfile - Check disk space: %s Port %d\n", ToBrowser.GetHost(), ToBrowser.GetPort());
             return -130;
+          }
         }
 
         TransferData = BodyQueue.begin();
 
-        if ( KEEPBACKBUFFER < (TransferDataLength - TransferData->length()) )
+        //Trickling
+        if ((TricklingTime != 0) && (LastTrickling + TricklingTime < time(NULL)))
+        {
+            //Send header only once
+            if ( HeaderSend == false )
+            {
+                HeaderSend  = true;
+                if( ToBrowser.SendHeader(&Header) == false)
+                {
+                  BrowserDropped = true;
+                  LogFile::ErrorMessage("Could not send Header to Browser: %s Port %d\n", ToBrowser.GetHost(), ToBrowser.GetPort());
+                  return -135;
+                }
+            }
+         LastTrickling = time(NULL);
+
+         BodyTemp = *TransferData;
+         TransferDataLength -= 1; //send one character
+         string character = TransferData->substr(0,1);
+         TransferData->erase(0,1);
+         if (ToBrowser.Send( &character ) == false)
+            {
+              BrowserDropped = true;
+              LogFile::ErrorMessage("Could not send Body to Browser: %s Port %d\n", ToBrowser.GetHost(), ToBrowser.GetPort());
+              return -138;
+            }
+
+         if ( TransferData->size() == 0 )
+         {
+            BodyQueue.erase( TransferData );
+         }
+        }
+
+
+        //Send Data
+        if ( KeepBackBuffer < (TransferDataLength - TransferData->length()) )
         {
 
 //This check will not work at the moment
@@ -328,10 +382,29 @@ int ProxyHandler::Communication( SocketHandler *ProxyServerT, GenericScanner *Vi
             BodyQueue.erase( TransferData );
         }
 
-    }
+      //End while because file is complete.
+      if ( ContentLength == ContentLengthReference ){
+        break;
+      }
+
+    }//while
+ } 
+
+// Check if there is some body e.g head
+//    if ( (ToBrowser.GetRequestType() != "HEAD") && (ContentLengthReference != 0) && (ScannerOff != true) && ( ContentLength != ContentLengthReference )){ 
+//            LogFile::ErrorMessage("ContentLength and Body size does not fit - body too short: %s Port %d\n", ToBrowser.GetHost(), ToBrowser.GetPort());;
+//      return -121;
+//    }
+
 
     //Wait till scanning is complete
     TempScannerAnswer =  VirusScannerT->ScanningComplete();
+
+    //Should we listen on the scanner output?
+    if (ScannerOff == true) {
+        TempScannerAnswer = 0;
+    }
+
     if ( TempScannerAnswer != 0)
     {
         #ifdef CATCHONSCANNERERROR
@@ -378,48 +451,90 @@ int ProxyHandler::Communication( SocketHandler *ProxyServerT, GenericScanner *Vi
 bool ProxyHandler::ProxyMessage( int CommunicationAnswerT , GenericScanner *VirusScannerT)
 {
 
-    string VirusError=VIRUSFOUND;
-    string ErrorHeader = ERRORHEADER;
-    string ScannerError = SCANNERERROR;
-    string DNSError = DNSERROR;
     string Answer;
 
+    long size;
+    char* buffer;
+    char ErrorNumber[11];
+    FileHandler fileH;
+    string filename;
+    string sendout="";
+    string message="";
+    string searchstring = "<!--message-->";
+
+
     //PSEstart
     //PSE: now get the answer from the second process
      Answer = VirusScannerT->ReadScannerAnswer();
     //PSEend
 
-    if (( HeaderSend == false ) && (BrowserDropped ==false ))
+    if (BrowserDropped ==true ){
+     LogFile::AccessMessage("%s Browser Droped: %d\n", ToBrowser.GetCompleteRequest(), CommunicationAnswerT);
+     return false;
+    }
+
+    if ( HeaderSend == false )
     {
-         LogFile::AccessMessage("%s Send Error Header: %d - PID: %d\n", ToBrowser.GetCompleteRequest(), CommunicationAnswerT, getpid());
-        ToBrowser.Send( &ErrorHeader );
+       string errorheader = ERRORHEADER;
+       if (ToBrowser.Send( &errorheader ) == false) {
+         LogFile::AccessMessage("%s Browser Droped: %d\n", ToBrowser.GetCompleteRequest(), CommunicationAnswerT);
+         return false;
+       }
     }
 
     if ( CommunicationAnswerT == -50 )
     {
         //Could not resolve DNS Name
         LogFile::AccessMessage("%s %d DNS Failed\n", ToBrowser.GetHost(), ToBrowser.GetPort());
-        ToBrowser.Send( &DNSError );
-
+        message = ToBrowser.GetHost();
+        filename = ERROR_DNS;
+    }
+    else if ( CommunicationAnswerT == -45 )
+    {
+        LogFile::AccessMessage("URL %s is blacklisted\n", ToBrowser.GetCompleteRequest() );
+        message = ToBrowser.GetCompleteRequest();
+        filename = ERROR_BLACKLIST;
     }
     else if ( CommunicationAnswerT == 2 )
     {
         LogFile::AccessMessage("%s %d Scanner Error: %s\n", ToBrowser.GetHost(), ToBrowser.GetPort(), Answer.c_str());
-        ScannerError.insert( ERRORINSERTPOSITION, Answer);
-        ToBrowser.Send( &ScannerError );
-
+        message = Answer;
+        filename = ERROR_SCANNER;
     }
     else if ( CommunicationAnswerT == 1 )
     {
         LogFile::AccessMessage("%s Virus: %s\n", ToBrowser.GetCompleteRequest(), Answer.c_str());
-        VirusError.insert( VIRUSINSERTPOSITION, Answer);
-        ToBrowser.Send( &VirusError );
+        message = Answer;
+        filename = VIRUS_FOUND;
     }
     else
     {
-        LogFile::AccessMessage("%s Error: %d\n", ToBrowser.GetCompleteRequest(), CommunicationAnswerT);
+        LogFile::AccessMessage("%s Error: %d - Status %d - Method %s\n", ToBrowser.GetCompleteRequest(), CommunicationAnswerT, ToServer.GetResponse(), ToBrowser.GetRequestType().c_str() );
+        snprintf(ErrorNumber, 10, "%d", CommunicationAnswerT);
+        message = ErrorNumber;
+        filename = ERROR_BODY;
     }
 
+
+    //Danny
+    if (filename.size() > 0) {
+      	string path=Params::GetConfigString("TEMPLATEPATH");
+      	filename = path + "/" + filename;
+        size = fileH.FileSize(filename.c_str());
+	if(size) {  // size=0 if error on open
+        buffer = new char[size+1];
+        fileH.FileRead(filename.c_str(), buffer, size);
+        buffer[size] = '\0';
+        sendout = buffer;
+
+        if (message.size() > 0) {
+            fileH.SearchReplace(sendout, searchstring, message);
+        } 
+        ToBrowser.Send( &sendout );
+        delete buffer;
+	}
+    }
+    
     return false;
 }
 
