@@ -18,9 +18,11 @@
 #ifndef SCANNERFILEHANDLER_H
 #define SCANNERFILEHANDLER_H
 
+
 #include "genericscanner.h"
 #include "logfile.h"
 #include "default.h"
+#include "params.h"
 
 //#include <stdio.h>
 //#include <stdlib.h>
@@ -30,7 +32,7 @@
 //#include <sys/types.h>
 //#include <sys/stat.h>
 #include <fcntl.h>
-
+#include  <errno.h>
 #include <iostream>
 
 using namespace std;
@@ -62,6 +64,10 @@ bool InitDatabase();
 
 bool ReloadDatabase();
 
+//PSEstart
+bool FreeDatabase();
+//PSEend
+
 int Scanning ();
 
 bool InitSelfEngine();
