@@ -20,6 +20,7 @@
 #include <arpa/inet.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <signal.h>
 #include "sockethandler.h"
 
 //Create Server Socket
@@ -50,34 +51,42 @@ bool SocketHandler::CreateServer( int portT, in_addr_t bind_addrT )
 
 
 //Create Server Socket, convert ASCII address representation into binary one
-bool SocketHandler::CreateServer( int portT, const char *bind_addrT )
+bool SocketHandler::CreateServer( int portT, string bind_addrT )
 {
-    return CreateServer( portT, bind_addrT == NULL ? INADDR_ANY : inet_addr( bind_addrT ) );
+    return CreateServer( portT, bind_addrT == "NULL" ? INADDR_ANY : inet_addr( bind_addrT.c_str() ) );
 }
 
 
 //Connect to Server
 bool SocketHandler::ConnectToServer (  )
 {
-    #if defined (SOURCE_ADDRESS)
-    struct sockaddr_in l_addr;
-    #endif
+    // #if defined (SOURCE_ADDRESS)
+    // struct sockaddr_in l_addr;
+    // #endif
 
     s_addr.sin_family = AF_INET;
 
     if ( (sock_fd = socket(s_addr.sin_family, SOCK_STREAM, 0)) == -1 )
         return false;
 
-    #if defined (SOURCE_ADDRESS)
+    // #if defined (SOURCE_ADDRESS)
+    string source_address=Params::GetConfigString("SOURCE_ADDRESS");
+    if(source_address != "") {
+    struct sockaddr_in l_addr;
     l_addr.sin_family = AF_INET;
-    l_addr.sin_addr.s_addr = inet_addr(SOURCE_ADDRESS);
+    l_addr.sin_addr.s_addr = inet_addr(source_address.c_str() );
     if ( ::bind ( sock_fd, (struct sockaddr *) &l_addr, sizeof ( l_addr ) ) == -1 )
         return false;
-    #endif
+    }
+    // #endif
 
     if ( ::connect(sock_fd, (struct sockaddr *) &s_addr, sizeof( s_addr ) ) == -1)
         return false;
 
+    FD_ZERO(&checkfd);
+    FD_SET(sock_fd,&checkfd);
+
+
     return true;
 }
 
@@ -90,10 +99,14 @@ bool SocketHandler::AcceptClient ( SocketHandler *accept_socketT )
 
     close ( sock_fd );
 
-    if ( accept_socketT->sock_fd == -1 )
+//PSE: Trigger a new process
+	  kill(getpgrp(),SIGUSR1);
+
+    if ( accept_socketT->sock_fd == -1 ){
         return false;
-    else
+    } else {
         return true;
+    }
 }
 
 
@@ -103,12 +116,19 @@ bool SocketHandler::Send ( string *sock_outT )
 
     int buffer_count;
     string send_temp;
-    fd_set write_allow;
 
-    FD_ZERO( &write_allow );
-    FD_SET( sock_fd, &write_allow );
 
-    select ( sock_fd+1, NULL, &write_allow, NULL, NULL);
+    select ( sock_fd+1, NULL, &checkfd, NULL, NULL);
+
+    //Timeout is changed with select
+    Timeout.tv_sec = SENDTIMEOUT;
+    Timeout.tv_usec = 0;
+    FD_ZERO(&checkfd);
+    FD_SET(sock_fd,&checkfd);
+    if (select( sock_fd+1, NULL, &checkfd, NULL, &Timeout) == 0)
+    {
+     return false;
+    }
 
     buffer_count = ::send ( sock_fd, sock_outT->c_str(), sock_outT->size(), MSG_NOSIGNAL );
 
@@ -135,6 +155,16 @@ ssize_t SocketHandler::Recv ( string *sock_inT , bool sock_delT)
     char buffer [ MAXRECV + 1 ];
     ssize_t buffer_count;
 
+    //Timeout is changed with select
+    Timeout.tv_sec = RECVTIMEOUT;
+    Timeout.tv_usec = 0;
+    FD_ZERO(&checkfd);
+    FD_SET(sock_fd,&checkfd);
+    if (select( sock_fd+1 ,&checkfd, NULL, NULL, &Timeout) == 0)
+    {
+      return false;
+    }
+
     if ( sock_delT == true )
     {
         buffer_count = ::recv ( sock_fd, buffer, MAXRECV, 0 );
@@ -176,6 +206,15 @@ bool SocketHandler::RecvLength ( string *sock_inT , ssize_t sock_lengthT )
 
     while ( sock_lengthT > 0 )
     {
+        //Timeout is changed with select
+        Timeout.tv_sec = RECVTIMEOUT;
+        Timeout.tv_usec = 0;
+        FD_ZERO(&checkfd);
+        FD_SET(sock_fd,&checkfd);
+        if (select( sock_fd+1 ,&checkfd, NULL, NULL, &Timeout) == 0)
+        {
+          return false;
+        }
 
         buffer_count = ::recv ( sock_fd, buffer, rest, 0 );
 
@@ -233,28 +272,17 @@ bool SocketHandler::SetDomainAndPort(const char *domainT, int portT)
 bool SocketHandler::CheckForData( )
 {
     fd_set checkfd;
-    struct timeval Timeout;
+
+
     Timeout.tv_sec = 0;
     Timeout.tv_usec = 0;
-
-    //Enable nonblocking sockets
-    fcntl(sock_fd, F_SETFL, O_NONBLOCK);
-
     FD_ZERO(&checkfd);
     FD_SET(sock_fd,&checkfd);
-
-    select( sock_fd+1 ,&checkfd, NULL, NULL, &Timeout);
-
     if (select( sock_fd+1 ,&checkfd, NULL, NULL, &Timeout) == 0)
     {
-        //Disable nonblocking sockets
-        fcntl(sock_fd, F_SETFL, ~O_NONBLOCK);
         return false;
     }
 
-    //Disable nonblocking sockets
-    fcntl(sock_fd, F_SETFL, ~O_NONBLOCK);
-
     return true;
 }
 
